/**
 * Copyright (c) 2016 - 2020, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#ifndef NRF_BLOCK_DEV_SPI_H__
#define NRF_BLOCK_DEV_SPI_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "nrf_block_dev.h"
#include "nrf_drv_spi.h"
#include "nrf_log_instance.h"

/**@file
 *
 * @defgroup nrf_block_dev_spi SPI implementation
 * @ingroup nrf_block_dev
 * @{
 *
 */

/**
 * @brief SPI block device operations
 * */
extern const nrf_block_dev_ops_t nrf_block_device_spi_ops;

/**
 * @brief SPI block device internal erase unit buffer size
 * */
#define NRF_BLOCK_DEV_SPI_ERASE_UNIT_SIZE (4096)

/**
 * @brief Internal Block device state
 */
typedef enum {
    NRF_BLOCK_DEV_SPI_STATE_DISABLED = 0,  /**< SPI block device state DISABLED      */
    NRF_BLOCK_DEV_SPI_STATE_IDLE,          /**< SPI block device state IDLE          */
    NRF_BLOCK_DEV_SPI_STATE_READ_EXEC,     /**< SPI block device state READ_EXEC     */
    NRF_BLOCK_DEV_SPI_STATE_EUNIT_LOAD,    /**< SPI block device state EUNIT_LOAD    */
    NRF_BLOCK_DEV_SPI_STATE_WRITE_ERASE,   /**< SPI block device state WRITE_ERASE   */
    NRF_BLOCK_DEV_SPI_STATE_WRITE_EXEC,    /**< SPI block device state WRITE_EXEC    */
} nrf_block_dev_spi_state_t;

/**
 * @brief Work structure of SPI block device
 */
typedef struct {
    volatile nrf_block_dev_spi_state_t state;        //!< SPI block device state

    nrf_block_dev_geometry_t geometry;                //!< Block device geometry
    nrf_block_dev_ev_handler ev_handler;              //!< Block device event handler
    void const *             p_context;               //!< Context handle passed to event handler
    nrf_block_req_t          req;                     //!< Block READ/WRITE request: original value
    nrf_block_req_t          left_req;                //!< Block READ/WRITE request: left value

    bool     cache_flushing;                                        //!< SPI cache flush in progress flag
    bool     writeback_mode;                                        //!< SPI write-back mode flag
    bool     erase_required;                                        //!< SPI erase required flag
    uint32_t erase_unit_idx;                                        //!< SPI erase unit index
    uint32_t erase_unit_dirty_blocks;                               //!< SPI erase unit dirty blocks mask
    uint8_t  p_erase_unit_buff[NRF_BLOCK_DEV_SPI_ERASE_UNIT_SIZE]; //!< SPI erase unit buffer (fixed value)
} nrf_block_dev_spi_work_t;

/**
 * @brief SPI block device flags*/
typedef enum {
    NRF_BLOCK_DEV_SPI_FLAG_CACHE_WRITEBACK = (1u << 0)  //!< Cache write-back mode enable flag
} nrf_block_dev_spi_flag_t;

/** @brief Name of the module used for logger messaging.
 */
#define NRF_BLOCK_DEV_SPI_LOG_NAME block_dev_spi

/**
 * @brief SPI block device config initializer (@ref nrf_block_dev_spi_config_t)
 *
 * @param blk_size          Block size
 * @param blk_flags         Block device flags, @ref nrf_block_dev_spi_flag_t
 * @param spi_drv_config   QPSI driver config
 * */
#define NRF_BLOCK_DEV_SPI_CONFIG(blk_size, blk_flags, spi_drv_config)  {   \
        .block_size = (blk_size),                                            \
        .flags = (blk_flags),                                                \
        .spi_config =  spi_drv_config                                      \
}

/**
 * @brief SPI block device config
 */
typedef struct {
    uint32_t block_size;                  //!< Desired block size
    uint32_t flags;                       //!< SPI block device flags
    nrf_drv_spi_config_t spi_config;    //!< SPI configuration
} nrf_block_dev_spi_config_t;

/**
 * @brief SPI block device
 * */
typedef struct {
    nrf_block_dev_t              block_dev;        //!< Block device
    nrf_block_dev_info_strings_t info_strings;     //!< Block device information strings
    nrf_block_dev_spi_config_t  spi_bdev_config; //!< SPI block device config
    nrf_block_dev_spi_work_t *  p_work;           //!< SPI block device work structure
    NRF_LOG_INSTANCE_PTR_DECLARE(p_log)            //!< Pointer to instance of the logger object (Conditionally compiled).
} nrf_block_dev_spi_t;

/**
 * @brief Defines a SPI block device.
 *
 * @param name    Instance name
 * @param config  Configuration @ref nrf_block_dev_spi_config_t
 * @param info    Info strings @ref NFR_BLOCK_DEV_INFO_CONFIG
 * */
#define NRF_BLOCK_DEV_SPI_DEFINE(name, config, info)                                            \
    static nrf_block_dev_spi_work_t CONCAT_2(name, _work);                                      \
    NRF_LOG_INSTANCE_REGISTER(NRF_BLOCK_DEV_SPI_LOG_NAME, name,                                 \
                              NRF_BLOCK_DEV_SPI_CONFIG_INFO_COLOR,                              \
                              NRF_BLOCK_DEV_SPI_CONFIG_DEBUG_COLOR,                             \
                              NRF_BLOCK_DEV_SPI_CONFIG_LOG_INIT_FILTER_LEVEL,                   \
                              NRF_BLOCK_DEV_SPI_CONFIG_LOG_ENABLED ?                            \
                                   NRF_BLOCK_DEV_SPI_CONFIG_LOG_LEVEL : NRF_LOG_SEVERITY_NONE); \
    static const nrf_block_dev_spi_t name = {                                                   \
            .block_dev = { .p_ops = &nrf_block_device_spi_ops },                                \
            .info_strings = BRACKET_EXTRACT(info),                                               \
            .spi_bdev_config = config,                                                          \
            .p_work = &CONCAT_2(name, _work),                                                    \
            NRF_LOG_INSTANCE_PTR_INIT(p_log, NRF_BLOCK_DEV_SPI_LOG_NAME, name)                  \
    }

/**
 * @brief Returns block device API handle from SPI block device.
 *
 * @param[in] p_blk_spi SPI block device
 * @return Block device handle
 */
static inline nrf_block_dev_t const *
nrf_block_dev_spi_ops_get(nrf_block_dev_spi_t const * p_blk_spi)
{
    return &p_blk_spi->block_dev;
}



typedef enum
{
    NRF_SPI_CINSTR_LEN_1B = (1UL), /**< Send opcode only. */
    NRF_SPI_CINSTR_LEN_2B = (2UL), /**< Send opcode, CINSTRDAT0.BYTE0. */
    NRF_SPI_CINSTR_LEN_3B = (3UL), /**< Send opcode, CINSTRDAT0.BYTE0 -> CINSTRDAT0.BYTE1. */
    NRF_SPI_CINSTR_LEN_4B = (4UL), /**< Send opcode, CINSTRDAT0.BYTE0 -> CINSTRDAT0.BYTE2. */
    NRF_SPI_CINSTR_LEN_5B = (5UL), /**< Send opcode, CINSTRDAT0.BYTE0 -> CINSTRDAT0.BYTE3. */
    NRF_SPI_CINSTR_LEN_6B = (6UL), /**< Send opcode, CINSTRDAT0.BYTE0 -> CINSTRDAT1.BYTE4. */
    NRF_SPI_CINSTR_LEN_7B = (7UL), /**< Send opcode, CINSTRDAT0.BYTE0 -> CINSTRDAT1.BYTE5. */
    NRF_SPI_CINSTR_LEN_8B = (8UL), /**< Send opcode, CINSTRDAT0.BYTE0 -> CINSTRDAT1.BYTE6. */
    NRF_SPI_CINSTR_LEN_9B = (9UL)  /**< Send opcode, CINSTRDAT0.BYTE0 -> CINSTRDAT1.BYTE7. */
} nrf_spi_cinstr_len_t;


typedef struct
{
    uint8_t               opcode;    /**< Opcode used in custom instruction transmission. */
    nrf_spi_cinstr_len_t length;    /**< Length of the custom instruction data. */
    bool                  io2_level; /**< I/O line level during transmission. */
    bool                  io3_level; /**< I/O line level during transmission. */
    bool                  wipwait;   /**< Wait if a Wait in Progress bit is set in the memory status byte. */
    bool                  wren;      /**< Send write enable before instruction. */
} nrf_spi_cinstr_conf_t;

typedef enum
{
    NRF_SPI_ERASE_LEN_4KB  = 0x20,  /**< Erase 4 kB block (flash command 0x20). */
    NRF_SPI_ERASE_LEN_64KB = 0xD8, /**< Erase 64 kB block (flash command 0xD8). */
    NRF_SPI_ERASE_LEN_ALL  = 0xC7   /**< Erase all (flash command 0xC7). */
} nrf_spi_erase_len_t;



void at_program_1(uint8_t * data_ptr,uint32_t data_length,uint32_t address);
ret_code_t spi_block_write(void *   p_tx_buffer,
                          size_t   tx_buffer_length,
                          uint32_t dst_address);

uint8_t SFlash_Erase(nrf_spi_erase_len_t length,uint32_t start_address);

/** @} */

#ifdef __cplusplus
}
#endif

#endif /* NRF_BLOCK_DEV_SPI_H__ */
